/*
 Copyright (C) 2009-2013 wxLauncher Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "global/ModIniKeys.h"

const wxString MOD_INI_KEY_LAUNCHER_MOD_NAME				(_T("/launcher/modname"));
const wxString MOD_INI_KEY_LAUNCHER_IMAGE_255X112			(_T("/launcher/image255x112"));
const wxString MOD_INI_KEY_LAUNCHER_IMAGE_182X80			(_T("/launcher/image182x80"));
const wxString MOD_INI_KEY_LAUNCHER_INFO_TEXT				(_T("/launcher/infotext"));
const wxString MOD_INI_KEY_LAUNCHER_AUTHOR					(_T("/launcher/author"));
const wxString MOD_INI_KEY_LAUNCHER_NOTES					(_T("/launcher/notes"));
const wxString MOD_INI_KEY_LAUNCHER_WARN					(_T("/launcher/warn"));

const wxString MOD_INI_KEY_LAUNCHER_WEBSITE					(_T("/launcher/website"));
const wxString MOD_INI_KEY_LAUNCHER_FORUM					(_T("/launcher/forum"));
const wxString MOD_INI_KEY_LAUNCHER_BUGS					(_T("/launcher/bugs"));
const wxString MOD_INI_KEY_LAUNCHER_SUPPORT					(_T("/launcher/support"));

const wxString MOD_INI_KEY_RESOLUTION_MIN_HORIZONTAL_RES	(_T("/resolution/minhorizontalres"));
const wxString MOD_INI_KEY_RESOLUTION_MIN_VERTICAL_RES		(_T("/resolution/minverticalres"));

const wxString MOD_INI_KEY_EXTREMEFORCE_FORCED_FLAGS_ON		(_T("/extremeforce/forcedflagson"));
const wxString MOD_INI_KEY_EXTREMEFORCE_FORCED_FLAGS_OFF	(_T("/extremeforce/forcedflagsoff"));

const wxString MOD_INI_KEY_MULTIMOD_PRIMARY_LIST			(_T("/multimod/primarylist"));
const wxString MOD_INI_KEY_MULTIMOD_SECONDRY_LIST			(_T("/multimod/secondrylist"));
const wxString MOD_INI_KEY_MULTIMOD_SECONDARY_LIST			(_T("/multimod/secondarylist"));

const wxString MOD_INI_KEY_SKIN_WINDOW_TITLE				(_T("/skin/windowtitle"));
const wxString MOD_INI_KEY_SKIN_WINDOW_ICON					(_T("/skin/windowicon"));
const wxString MOD_INI_KEY_SKIN_BANNER						(_T("/skin/banner"));
const wxString MOD_INI_KEY_SKIN_WELCOME_TEXT				(_T("/skin/welcometext"));

const wxString MOD_INI_KEY_SKIN_MOD_IMAGE_255X112			(_T("/skin/modimage255x112"));
const wxString MOD_INI_KEY_SKIN_MOD_IMAGE_182X80			(_T("/skin/modimage182x80"));

const wxString MOD_INI_KEY_SKIN_ICON_OK						(_T("/skin/iconok"));
const wxString MOD_INI_KEY_SKIN_ICON_WARNING				(_T("/skin/iconwarn"));
const wxString MOD_INI_KEY_SKIN_ICON_WARNING_BIG			(_T("/skin/iconwarnbig"));
const wxString MOD_INI_KEY_SKIN_ICON_ERROR					(_T("/skin/iconerror"));
const wxString MOD_INI_KEY_SKIN_ICON_INFO					(_T("/skin/iconinfo"));
const wxString MOD_INI_KEY_SKIN_ICON_INFO_BIG				(_T("/skin/iconinfobig"));
const wxString MOD_INI_KEY_SKIN_ICON_HELP					(_T("/skin/iconhelp"));
const wxString MOD_INI_KEY_SKIN_ICON_HELP_BIG				(_T("/skin/iconhelpbig"));
const wxString MOD_INI_KEY_SKIN_ICON_IDEAL					(_T("/skin/iconideal"));

const wxString MOD_INI_KEY_SKIN_NEWS_SOURCE					(_T("/skin/newssource"));

const wxString MOD_INI_KEY_RECOMMENDED_LIGHTING_NAME		(_T("/recommendedlighting/name"));
const wxString MOD_INI_KEY_RECOMMENDED_LIGHTING_FLAGSET		(_T("/recommendedlighting/flagset"));
