/*
 Copyright (C) 2009-2013 wxLauncher Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef SKIN_DEFAULTS_H
#define SKIN_DEFAULTS_H

#include <wx/string.h>
#include "datastructures/NewsSource.h"

/** \defgroup skindefaults values for default skin */
/** @{*/
extern const wxString DEFAULT_SKIN_WINDOW_TITLE;
extern const wxString DEFAULT_SKIN_WINDOW_ICON;
extern const wxString DEFAULT_SKIN_BANNER;
extern const wxString DEFAULT_SKIN_WELCOME_TEXT;

extern const wxString DEFAULT_SKIN_MOD_IMAGE_255X112;
extern const wxString DEFAULT_SKIN_MOD_IMAGE_182X80;

extern const wxString DEFAULT_SKIN_ICON_OK;
extern const wxString DEFAULT_SKIN_ICON_WARNING;
extern const wxString DEFAULT_SKIN_ICON_WARNING_BIG;
extern const wxString DEFAULT_SKIN_ICON_ERROR;
extern const wxString DEFAULT_SKIN_ICON_INFO;
extern const wxString DEFAULT_SKIN_ICON_INFO_BIG;
extern const wxString DEFAULT_SKIN_ICON_HELP;
extern const wxString DEFAULT_SKIN_ICON_HELP_BIG;
extern const wxString DEFAULT_SKIN_ICON_IDEAL;

extern const NewsSourceId DEFAULT_SKIN_NEWS_SOURCE;
/** @}*/

#endif
