/*
 Copyright (C) 2009-2013 wxLauncher Team
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "SkinDefaults.h"
#include <wx/intl.h> // for _() macro

const wxString DEFAULT_SKIN_WINDOW_TITLE		(_("wxLauncher for the FreeSpace 2 Source Code Project"));
const wxString DEFAULT_SKIN_WINDOW_ICON			(_T("wxlauncher.ico"));
const wxString DEFAULT_SKIN_BANNER				(_T("SCP_Header.png"));
const wxString DEFAULT_SKIN_WELCOME_TEXT
	(_("<p><center><b><font size='3'>Welcome to wxLauncher, the next-generation launcher for FS2 Open games</font></b><br><br>New to FreeSpace 2? Check out these links:<br><br> = <a href='http://www.hard-light.net/wiki/index.php/Main_Page'>FreeSpace Wiki</a> = <a href='http://www.hard-light.net/forums/'>Hard Light Productions Forums</a> = <br> = <a href='http://www.hard-light.net/forums/index.php?topic=63609.0'>Multiplayer Setup</a> = <a href='http://www.hard-light.net/forums/index.php?board=151.0'>Support Forum</a> = <br> = <a href='http://www.hard-light.net/forums/index.php?topic=56279.0'>Troubleshooting FAQ</a> = </center></p>"));

const wxString DEFAULT_SKIN_MOD_IMAGE_255X112	(_T("modimg.png"));
const wxString DEFAULT_SKIN_MOD_IMAGE_182X80	(_T("modimg_small.png"));

const wxString DEFAULT_SKIN_ICON_OK				(_T("icon_ok.png"));
const wxString DEFAULT_SKIN_ICON_WARNING		(_T("icon_warning.png"));
const wxString DEFAULT_SKIN_ICON_WARNING_BIG	(_T("warning_big.png"));
const wxString DEFAULT_SKIN_ICON_ERROR			(_T("icon_error.png"));
const wxString DEFAULT_SKIN_ICON_INFO			(_T("icon_information.png"));
const wxString DEFAULT_SKIN_ICON_INFO_BIG		(_T("info_big.png"));
const wxString DEFAULT_SKIN_ICON_HELP			(_T("helpicon.png"));
const wxString DEFAULT_SKIN_ICON_HELP_BIG		(_T("questionmark.png"));
const wxString DEFAULT_SKIN_ICON_IDEAL			(_T("recommended.png"));

const NewsSourceId DEFAULT_SKIN_NEWS_SOURCE		(NEWS_SOURCE_ID_HLP);
