/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.

IDI_WXLAUNCHER       ICON         "wxlauncher.ico"
IDI_SMALL               ICON         "wxlauncher.ico"
